@@ -21,11 +21,11 @@ import binascii
 
 from cubicweb import RegistryException
 from cubicweb.server import hook
-from cubicweb.selectors import is_instance, on_fire_transition
+from cubicweb.selectors import is_instance, on_fire_transition, has_related_entities
 from cubicweb.hooks import notification
 
 from cubes.vcsfile import bridge
-from cubes.vcreview import users_by_author
+from cubes.vcreview import users_by_author, compare_patches
 
 
 def dead_revision(rev):
@@ -70,6 +70,21 @@ class RevisionAdded(hook.Hook):
             operation = AttachToPatchesOp.get_instance(self._cw)
             operation.add_data((repo.eid, self.entity.eid))
 
+
+class RevisionDeleted(hook.Hook):
+    """When deleting a revision, delete the associated patch if it has
+    no other revisions
+    """
+    __regid__ = 'vcreview.revision_gone'
+    __select__ = (hook.Hook.__select__ & is_instance('Revision') &
+                  has_related_entities('patch_revision', role='object'))
+    events = ('before_delete_entity',)
+
+    def __call__(self):
+        patch = self.entity.reverse_patch_revision[0]
+        PatchLifeCheckOp.get_instance(self._cw).add_data((self.entity.repository.eid, patch.eid, None))
+
+
 class AttachToPatchesOp(hook.DataOperationMixIn, hook.LateOperation):
     """Attach new revision to a Patch entity
 
@@ -165,6 +180,14 @@ class PatchLifeCheckOp(hook.DataOperationMixIn, hook.SingleLastOperation):
             # nothing to be done on it anymore
             if patch.is_final:
                 continue
+            if eid_rev is None:
+                rset = self.session.execute(
+                        'DELETE Patch P WHERE P eid %(eid)s,'
+                        '                     NOT EXISTS(P patch_revision R)',
+                        {'eid': patch.eid})
+                if rset:
+                    self.info('Deleted patch %s, no revisions left', rset[0][0])
+                continue
             rev = session.entity_from_eid(eid_rev)
             assert rev.reverse_patch_revision
             ### Case 2: patch has just been applied
@@ -179,8 +202,13 @@ class PatchLifeCheckOp(hook.DataOperationMixIn, hook.SingleLastOperation):
             tips_rset = patch.all_tips()
             visible_tips = set(r[0] for r in tips_rset if not r[1])
             if visible_tips:
-                if eid_rev not in visible_tips:
-                    # we do not care about this revision
+                # pre-existing revisions are likely just set to hidden=True
+                # in this transaction. We do not want them to affect patch
+                # state if we have a new visible tip (or even newer revision).
+                if not (session.added_in_transaction(eid_rev)
+                        or eid_rev in visible_tips):
+                    continue
+                if self._is_rebase(rev, wfable.state):
                     continue
                 ### Case 3: new version of the patch
                 if wfable.state != 'pending-review':
@@ -234,6 +262,20 @@ class PatchLifeCheckOp(hook.DataOperationMixIn, hook.SingleLastOperation):
                         toproceed.add(s)
         return None
 
+    def _is_rebase(self, rev, patch_state):
+        """revision looks like a rebase and the patch state should be kept"""
+        if rev.obsoletes:
+            # preserve useful state only. we do not want to keep patch in
+            # the "outdated" state.
+            # XXX We may want to detect outdated patches and restore their
+            # previous state, but that is for later
+            if patch_state in ('in-progress', 'pending-review', 'reviewed'):
+                content = rev.export()
+                if all(compare_patches(old.export(), content)
+                       for old in rev.obsoletes):
+                    return True
+        return False
+
 
 class InitPatchCreatorHook(hook.Hook):
     """automatically add people to nosy list
@@ -309,3 +351,4 @@ class ReviewerSetHook(hook.Hook):
 
 from cubes.nosylist import hooks as nosylist
 nosylist.S_RELS |= set(('has_activity', 'patch_revision'))
+nosylist.O_RELS |= set(('point_of', 'comments'))
