@@ -47,5 +47,6 @@ def insert_point_callback(ipid, trdata, w):
             comp.render(w=w)
     w(u'</div>')
 
-ENGINE.remove_transform('Diff2HTMLTransform', 'text/x-diff')
+if Diff2HTMLTransform.name in ENGINE.transforms:
+    ENGINE.remove_transform(Diff2HTMLTransform.name, 'text/x-diff')
 ENGINE.add_transform(Diff2HTMLTransform(insert_point_callback))
