@@ -3,6 +3,8 @@ from cubicweb.selectors import is_instance, is_in_state
 from cubicweb.sobjects.notification import StatusChangeMixIn, NotificationView
 from cubes.task import hooks as task
 
+MAX_PATCH_SIZE = 100 << 10 # 100 Ko
+
 class PatchStatusChangeView(StatusChangeMixIn, NotificationView):
     __select__ = is_instance('Patch') & is_in_state('pending-review',
                                                     'in-progress',
@@ -10,6 +12,11 @@ class PatchStatusChangeView(StatusChangeMixIn, NotificationView):
                                                     'applied',
                                                     'rejected')
 
+    content = StatusChangeMixIn.content + '''
+
+%(patch)s
+'''
+
     def subject(self):
         entity = self.cw_rset.get_entity(0, 0)
         return self._cw._(u'[%(repo)s] patch %(state)s: %(patch)s') % {
@@ -39,6 +46,13 @@ class PatchStatusChangeView(StatusChangeMixIn, NotificationView):
         else:
             context['comment'] += '\n\n%s' % self._cw._('(No recorded task)')
 
+        context['patch'] = ''
+        if self._kwargs['current_state'] == 'pending-review':
+            patch = entity.tip().export()
+            if len(patch) > MAX_PATCH_SIZE:
+                patch = patch[:MAX_PATCH_SIZE] + '\n@@ truncated patch @@'
+            context['patch'] = patch
+
         return context
 
 
@@ -53,9 +67,10 @@ class TaskAddedView(task.TaskAddedView):
         if not patch:
             return super(TaskAddedView, self).subject()
         if patch.__regid__ == 'InsertionPoint':
-            patch = patch.parent.patch
+            patch = patch.parent.reverse_patch_revision
             if not patch:
                 return super(TaskAddedView, self).subject()
+            patch = patch[0]
         return self._cw._(u'[%(repo)s] patch task: %(patch)s - %(task)s') % {
             'repo': patch.repository.dc_title(),
             'patch': patch.dc_title(),
@@ -69,6 +84,8 @@ class ReviewerSetNotificationView(NotificationView):
 You have been assigned for review of patch %(title)s
 
 url: %(url)s
+
+%(patch)s
 ''')
     def recipients(self):
         reviewer = self.cw_extra_kwargs['user']
@@ -84,6 +101,15 @@ url: %(url)s
             'repo': patch.repository.dc_title(),
             'patch': patch.dc_title()}
 
+    def context(self, **kwargs):
+        context = super(ReviewerSetNotificationView, self).context(**kwargs)
+        entity = self.cw_rset.get_entity(0, 0)
+        patch = entity.tip().export()
+        if len(patch) > MAX_PATCH_SIZE:
+            patch = patch[:MAX_PATCH_SIZE] + '\n@@ truncated patch @@'
+        context['patch'] = patch
+        return context
+
 
 def registration_callback(vreg):
     vreg.register_all(globals().values(), __name__, (TaskAddedView,))
