@@ -36,7 +36,7 @@ def revnums(patch):
 
 
 
-class PatchLifeCycleTC(CubicWebTC):
+class HgHelper(CubicWebTC):
 
     verbose_hg = False
 
@@ -50,6 +50,7 @@ class PatchLifeCycleTC(CubicWebTC):
                                                   path=unicode(self.tmprepo),
                                                   has_review=True)
         self.refresh()
+        self.session.commit(free_cnxset=False)
 
     def tearDown(self):
         bridge._REPOHDLRS.clear()
@@ -57,7 +58,7 @@ class PatchLifeCycleTC(CubicWebTC):
         repo = getattr(self, 'cwhgrepo', None)
         if getattr(repo, 'local_cache', None) is not None:
             rmtree(repo.local_cache, ignore_errors=True)
-        return super(PatchLifeCycleTC, self).tearDown()
+        return super(HgHelper, self).tearDown()
 
     def call_hg(self, *args):
         """Call a mercurial command in the repo"""
@@ -105,6 +106,8 @@ class PatchLifeCycleTC(CubicWebTC):
         return [tr.new_state.name
                 for tr in patch.cw_adapt_to('IWorkflowable').workflow_history]
 
+class PatchLifeCycleTC(HgHelper):
+
     def test_root_publishing(self):
 
         hg = self.call_hg
@@ -347,6 +350,27 @@ class PatchLifeCycleTC(CubicWebTC):
                          wf_history)
         self.assertEqual(2, len(patch.patch_revision))
 
+    def test_delete_revision(self):
+        self.test_root_publishing()
+        hg = self.call_hg
+        a = self.append_to
+        oldpatch = self.all_revs()[0].reverse_patch_revision[0]
+        # create a new draft child
+        a('jungle', 'Flare')
+        hg('commit', '-m', 'second child')
+        self.refresh()
+
+        rev = self.all_revs()[-1]
+        assert rev.description == 'second child'
+        self.assertTrue(rev.reverse_patch_revision)
+        eid_patch = rev.reverse_patch_revision[0].eid
+        with self.repo.internal_session() as session:
+            self.assertTrue(session.execute('DELETE Revision R WHERE R eid %(eid)s',
+                                            {'eid': rev.eid}))
+            session.commit()
+        self.assertFalse(self.execute('Patch P WHERE P eid %(eid)s',
+                                      {'eid': eid_patch}))
+
     def test_split(self):
         self.test_root_publishing()
         hg = self.call_hg
@@ -513,6 +537,7 @@ class PatchNotificationTC(CubicWebTC):
         # we should print a diff like -/+ stuff
         for idx, line in enumerate(expected):
             self.assertMatches(line, got[idx])
+        self.assertEqual(len(expected), len(got))
 
     def assertMatches(self, expected, got):
         """ expected is a regular expression """
@@ -552,34 +577,62 @@ class PatchNotificationTC(CubicWebTC):
                                                            u'coin coin')
         session.commit(free_cnxset=False)
         self.assertEqual(2, len(MAILBOX))
+        MAILBOX[:] = []
+        # patch are only send on ask review, so we reask for review
+        patch.cw_adapt_to('IWorkflowable').fire_transition('ask review',
+                                                           u'coin coin')
+        session.commit(free_cnxset=False)
+        self.assertEqual(2, len(MAILBOX))
         MAILBOX.sort(key=lambda e: 'victor@ratax.es' in e.recipients)
         review_email = MAILBOX[0]
         self.assertEmailEqual(review_email,
-                              '[Jungle] patch in-progress: Add more elephants children',
+                              '[Jungle] patch pending-review: Add more elephants children',
                               [e.address for e in reviewer.primary_email][0],
                               '\n'
-                              'admin changed status from <pending-review> to <in-progress> for entity\n'
+                              'admin changed status from <in-progress> to <pending-review> for entity\n'
                               "'Add more elephants children'\n"
                               '\n'
                               'coin coin\n'
                               '\n'
                               '\(No recorded task\)\n'
                               '\n'
-                              'url: http://testing.fr/cubicweb/patch/\d+\n')
-        # reask for review
-        MAILBOX[:] = ()
-        patch.cw_adapt_to('IWorkflowable').fire_transition('ask review',
-                                                           u'please do')
-        session.commit(free_cnxset=False)
-        self.assertEqual(2, len(MAILBOX))
-        MAILBOX[:] = ()
+                              'url: http://testing.fr/cubicweb/patch/\d+\n'
+                              '\n'
+                              '\n'
+                              '# HG changeset patch\n'
+                              '# User Victor <victor@ratax.es>\n'
+                              '# Date 1361805588 0\n'
+                              '# Node ID bfa5da61c82507b76dae7927b4fce2aa4a0c9c6f\n'
+                              '# Parent  549cd3165597cdd5c9d8fa26e83e63483fcfb4b5\n'
+                              'Add more elephants children\n'
+                              '\n'
+                              'diff --git a/babar.py b/babar.py\n'
+                              '--- a/babar.py\n'
+                              '\+\+\+ b/babar.py\n'
+                              '@@ -5,10 \+5,15 @@\n'
+                              " elephants = \['Babar',\n"
+                              "              'Celeste',\n"
+                              "              'Cornelius',\n"
+                              "              'Pompadour',\n"
+                              "              'Poutifour',\n"
+                              "\+             'Arthur'\n"
+                              "\+             'Pom',\n"
+                              "\+             'Flore',\n"
+                              "\+             'Alexandre',\n"
+                              "\+             'Isabelle',\n"
+                              '              \]\n'
+                              ' \n'
+                              ' \n'
+                              " if __name__ == '__main__':\n"
+                              "     print 'Those are elephants:'\n"
+                              '\n')
         # add task
         MAILBOX[:] = ()
-        session.create_entity('Task', title=u'todo', reverse_has_activity=patch)
+        task = session.create_entity('Task', title=u'todo', reverse_has_activity=patch)
         point = session.create_entity('InsertionPoint', lid=0, point_of=rev)
         session.create_entity('Task', title=u'my point', reverse_has_activity=point)
         session.commit(free_cnxset=False)
-        self.assertEqual(2, len(MAILBOX))
+        self.assertEqual(4, len(MAILBOX))
         # reask review
         MAILBOX[:] = ()
         patch.cw_adapt_to('IWorkflowable').fire_transition('ask rework',
@@ -602,7 +655,27 @@ class PatchNotificationTC(CubicWebTC):
                               '- my point\n'
                               '- todo\n'
                               '\n'
-                              'url: http://testing.fr/cubicweb/patch/\d+\n')
+                              'url: http://testing.fr/cubicweb/patch/\d+\n'
+                              '\n'
+                              '\n'
+                              '\n')
+
+        # Test that comments are emailed too
+        MAILBOX[:] = ()
+        point.cw_clear_all_caches()
+        comment = session.create_entity('Comment', comments=task,
+                                        content=u'Babar comments')
+        session.commit(free_cnxset=False)
+        MAILBOX.sort(key=lambda e: 'victor@ratax.es' in e.recipients)
+        self.assertEqual(2, len(MAILBOX))
+        review_email = MAILBOX[0]
+        self.assertEmailEqual(review_email,
+                              'new comment for Task todo',
+                              [e.address for e in reviewer.primary_email][0],
+                              'Babar comments\n'
+                              '\n\n'
+                              'i18n_by_author_field: admin\n'
+                              'url: http://testing.fr/cubicweb/task/%i' % task.eid)
 
 
 PatchNotificationTC.repo_path = osp.join(PatchNotificationTC.datadir,
